# EMFAC2014_CO2Emissions2017-12-18.R 
# Sarah Grajdura 1/28/2019
# Reads EMFAC2017 Output http://www.arb.ca.gov/emfac/2017/ and 
# computes vehicle class and fuel adjusted CO2 grams per mile traveled
# and formats the output in a standard ITHIM format 

# set working directory
here <- getwd()
setwd <- here

# Get todays date to assign version
# today <- Sys.Date()
today <- date()

# turn off scientific notation
options(scipen=999)

# read in files 
files = list.files(path = "/Users/sgrajdu2/Box/ITHIM/EMFAC stuff/EMFAC Data")

# INITIALIZATION
# create empty file to add rows to 
CO2 <- data.frame(Region=character(), 
                  Year=double(), 
                  CO2g_mi=double()
                  )

# LOOP
for (i in 1:length(files)) {
  # load Excel file for a geography
  header <- read.csv(files[i], skip=8, nrows=1, header=FALSE, sep=",", stringsAsFactors = FALSE)
  emfac <- read.csv(files[i], skip=9, header=FALSE, sep=",")
  colnames(emfac) <- unlist(header)
  
  # keep var's of interest
  emfac <- emfac[, c("CalYr", "VehClass", "Fuel", "VMT", "Trips", "CO2_RUNEX", "CO2_STREX")]
  
  # keep years of interest 
  emfac <- emfac[emfac$CalYr %% 5 == 0,]
  
  # calc total CO2
  emfac$total_co2 <- emfac$CO2_RUNEX + ((emfac$CO2_STREX*emfac$Trips)/emfac$VMT)

  # get weighted mean for all years 
  tmp <- sapply(split(emfac, emfac$CalYr), function(d) weighted.mean(d$total_co2, w = d$VMT))
  tmp <- as.data.frame(tmp)
  tmp <- cbind(variable = row.names(tmp),tmp)
  
  # get geography name
  geography <- unlist(strsplit(files[i], split = "-"))
  
  # add geography name to df
  tmp$Region <- geography[4]
  
  # Rearrange columns 
  tmp$Year <- tmp$variable
  tmp$CO2g_mi <- tmp$tmp
  tmp <- tmp[3:5]
  
  # append info to final file 
  CO2 <- rbind(CO2, tmp)
}

# export file
write.csv(CO2,"emfac2014_sarah.csv", row.names=FALSE)



















