* NHTS2009_Woodcock_all_modes4-11-13.sas  N Maizlish, N Linesch, B Haydu  07/24/13 ;
* Read NHTS 2009 files and subsets urban California travel trips;
* Calculates daily per capita mean travel time and distances by mode;

* Read Person file;
* 2009;
proc import datafile= 'T:\ITHIM\OtherData\NHTS2009\DATA\PERV2PUB_CA.csv' dbms=dlm out=persons replace;
* proc import datafile= 'D:\Brandon\ITHIM\NHTS\Ascii\PERV2PUB.csv' dbms=dlm out=persons replace;
delimiter=",";
getnames=yes;
guessingrows=500;
run;


* Keep California cases only;
data persons (KEEP = HOUSEID PERSONID R_SEX R_AGE WTPERFIN HH_RACE);
  set persons;
  if HHSTATE ='CA';* and URBRUR = 1;
run;


proc import datafile= 'T:\ITHIM\Crosswalks\NHTS_HH_Lookup.csv' dbms=dlm out=lookup replace;
*proc import datafile= 'D:\Brandon\ITHIM\NHTS_HH_Lookup.csv' dbms=dlm out=lookup replace;

delimiter=",";
getnames=yes;
guessingrows=500;
run;

** Rename HouseID to HHID;
data lookup (KEEP = HOUSEID HHSTATE county chis_reg mpo cstdm_region megaregion);
  set lookup;*(RENAME=(HOUSEID=HHID));
run;

* Read Regional Look-up Tables;
proc import datafile= 'T:\ITHIM\Crosswalks\CA_geo2010.xls' dbms=xls out= CA_geo2010 replace;
getnames=yes;
run;

**sorts the lookup file by what you merge it with;
proc sort data=lookup;
by HOUSEID;
run;

proc sort data=persons;
by HOUSEID PERSONID;
run;

data persons;
merge persons (IN = recent) lookup;
BY HOUSEID;
IF recent = 1;
run;


data persons;
 set persons;
* Create age categories;
if (r_age>=0  AND r_age<=4)  then age_group = '00-04';
if (r_age>=5  AND r_age<=14) then age_group = '05-14';
if (r_age>=15 AND r_age<=29) then age_group = '15-29';
if (r_age>=30 AND r_age<=44) then age_group = '30-44';
if (r_age>=45 AND r_age<=59) then age_group = '45-59';
if (r_age>=60 AND r_age<=69) then age_group = '60-69';
if (r_age>=70 AND r_age<=79) then age_group = '70-79';
if (r_age>=80 AND r_age<=99) then age_group = '80+';

* Recode Sex;
if r_sex = 1 then sex = "M";
if r_sex = 2 then sex = "F";

* Recode Race;
if (hh_race=-1 OR hh_race=-7 OR hh_race=-8 OR hh_race=-9) then race="missing";
if (hh_race=1) then race="White";
if (hh_race=2) then race="Black";
if (hh_race=3) then race="Asian";
if (hh_race=4) then race="Native";
if (hh_race=5) then race="PacificIs";
if (hh_race=6) then race="Multi";
if (hh_race=7) then race="Latino";
if (hh_race=97) then race="Other";
run;

proc sort;
 by houseid personid;
run;

proc contents; run;

* Read Trip file;
* 2009 ;
proc import datafile= 'T:\ITHIM\OtherData\NHTS2009\DATA\DAYV2PUB_CA.csv' dbms=dlm out=trips replace;
delimiter=",";
getnames=yes;
guessingrows=500;
run;


proc format;
value TRPTRANSf /*TRPTRANS*/ 
	-1 = 'A skip'
	-7 = 'Refused'
	-8 = 'Don''t know'
	-9 = 'Not ascertained'
	01 = 'Car'
	02 = 'Van'
	03 = 'SUV'
	04 = 'Pickup truck'
	05 = 'Other truck'
	06 = 'RV'
	07 = 'Motorcycle'
	08 = 'Light electric veh (golf cart)'
	09 = 'Local public bus'
	10 = 'Commuter bus'
	11 = 'School bus'
	12 = 'Charter/tour bus'
	13 = 'City to city bus'
	14 = 'Shuttle bus'
	15 = 'Amtrak/inter city train'
	16 = 'Commuter train'
	17 = 'Subway/elevated train'
	18 = 'Street car/trolley'
	19 = 'Taxicab'
	20 = 'Ferry'
	21 = 'Airplane'
	22 = 'Bicycle'
	23 = 'Walk'
	24 = 'Special transit-people w/disabilities'
	97 = 'Other'
	;

* Subset for urban households; 
data trips (KEEP = HOUSEID PERSONID HHSTATE URBRUR TDTRPNUM TRPMILES TRVL_MIN TRPTRANS WTTRDFIN);
  set trips;
run;


* Subset trips in California urban households by mode; 
data trips;

* Next statement makes sure mode is not truncated;
format mode $CHAR12.;


set trips;

* Create mode;
* Bicycle;
if (TRPTRANS EQ 22) then mode = "BICYCLE";

* Pedestrian;
if (TRPTRANS EQ 23) then mode = "WALK";

* Car, Van, SUV, pick-up truck, taxi;
if ((TRPTRANS >= 1 AND TRPTRANS <=4) OR trptrans = 19) then mode = "CAR";

* Bus;
if (TRPTRANS GE 9 AND TRPTRANS LE 14) then mode = "BUS";

* Rail;
if (TRPTRANS GE 16 AND  TRPTRANS LE 18) then MODE = "RAIL";

* Motorcycle;
if (TRPTRANS EQ 7) then mode = "MOTORCYCLE";

* Other;
if (TRPTRANS EQ 97) then mode = "OTHER";

FORMAT trptrans trptransf.;
run;

* proc contents; run;

* Impute missing data;

data trips;
  set trips;

* Impute missing data from average speed for walking and bicycling.  Note: These are avgs across regions

* Bicycle = 22  10.8 mi/hr = 0.18 mi/min;
* Non-missing travel time, but missing trip miles;
if (trptrans = 22 and trpmiles <0 and trvl_min >0) then trpmiles = trvl_min*0.18;

* Missing travel time, but non-missing trip miles;
if (trptrans = 22 and trvl_min <0 and trpmiles >0) then trvl_min = trpmiles/0.18;

* Walk = 23  2.78 mi/hr = 0.046283 mi/min;
* Non-missing travel time, but missing trip miles;
if (trptrans = 23 and trpmiles <0 and trvl_min >0) then trpmiles = trvl_min*0.046283;

* Missing travel time, but non-missing trip miles;
if (trptrans = 23 and trvl_min <0 and trpmiles >0) then trvl_min = trpmiles/0.046283;

* Fix personID  21537341-1 (660 min to tral 1.67 miles replace w/aver);
if houseid = 21537341 AND personid = 1 then trvl_min = 1.67/0.046283;

if trpmiles < 0 then trpmiles_missing = 1;
if trvl_min < 0 then trl_min_missing  = 1;

run;

*Add a speed field to the trips file where miles and minutes are not missing;
data trips;
  set trips;
*calc. speeds for all trips (miles / (minutes/60) ) for trips with non missing values;
if (trpmiles > 0 AND trvl_min > 0 ) then speed = TRPMILES/(TRVL_MIN/60);
run;

*Join the field megaregion into the trips file;
proc sql;
CREATE TABLE trips AS
SELECT A.*, B.megaregion
FROM trips A LEFT OUTER JOIN  PERSONS B ON (A.HOUSEID=B.HOUSEID AND A.PERSONID=B.PERSONID);
*WHERE A.TRVL_MIN > 0 AND A.TRPMILES > 0 AND A.mode IS NOT NULL;
run;

PROC SORT;
BY megaregion mode;
RUN;

*Create table "avg_speed" which is the average speed by mode & megaregion;
PROC MEANS noprint;
BY megaregion mode;
VAR speed;
output out=avg_speed MEAN(speed)=AVG_SPEED;
run;

*Join the average speed calculated in the previous step back into the trips file;
proc sql;
CREATE TABLE trips AS
SELECT A.*, B.AVG_SPEED
FROM trips A INNER JOIN avg_speed B ON (A.mode=B.mode AND A.megaregion=B.megaregion);
run;

*Use average speed to inpute missing values;
*Impute missing data from average speed for bus trips.
*Note: These are average bus speeds calc from NHTS;

data trips;
set trips;
if (mode = "BUS" and trpmiles_missing = 1 and trvl_min_missing = .) then trpmiles = AVG_SPEED*trvl_min/60;
if (mode = "BUS" and trpmiles_missing = . and trvl_min_missing = 1) then trvl_min = trpmiles*60/AVG_SPEED;
RUN;

*Drop the newly created fields so the trips file returns to the original form;
proc sql;
CREATE TABLE trips AS
SELECT MODE, HOUSEID, PERSONID, HHSTATE, TRPTRANS, TRVL_MIN, URBRUR, TDTRPNUM, TRPMILES, WTTRDFIN, trpmiles_missing, trl_min_missing
FROM trips;
RUN;


* Frequency of Missing data;
proc freq data=trips;
   tables mode;
   tables trpmiles_missing trl_min_missing;
   tables trpmiles_missing*mode trl_min_missing*mode;
run; 

********** Beginning of looping macro for geography; 
%LET scenario = 1;

%macro geoloop(geolist);
%local j geography;
%let j=1;
%let geography=%scan(&geolist,&j);
%do %while (&geography ne);


********** Beginning of looping macro for mode; 
%macro loop1(modelist);
%local i modegroup;
%let i=1;
%let modegroup=%scan(&modelist,&i);
%do %while (&modegroup ne); 


* Subgroup trips by mode;
data trips&modegroup;
 set trips;
 if mode = "&modegroup";
run;

* If trips are unknown set to missing;
* Or, 0 minute/mile trips but min/miles >0 ;
data trips&modegroup;
 set trips&modegroup;
if trpmiles  < 0 then trpmiles  = .;
if trvl_min  < 0 then trvl_min  = .;
if trvl_min  = 0 and trpmiles >0 then trvl_min  = .;
if trpmiles  = 0 and trvl_min >0 then trpmiles  = .;
run;

* proc contents;run;

********* Aggregate Miles and Travel Time by Person and Mode **********;
proc sort;
 by houseid personid;
run;

* Aggregate miles and minutes over trips by person;
proc means noprint;
 by houseid personid;
 var trpmiles trvl_min;
output out = collapsed_sum_&modegroup SUM(trpmiles) = sum_miles SUM(trvl_min) = sum_minutes;
run;

proc sort; by houseid personid;run;


* Merge person file with bicycle and walk miles;
data person_miles_minutes_&modegroup;
 merge persons collapsed_sum_&modegroup;
  by houseid personid;
 run;


* daily miles and minutes for those not reporting trips;
data person_miles_minutes_&modegroup;
 set person_miles_minutes_&modegroup;
if sum_miles    = . then sum_miles   = 0;
if sum_minutes  = . then sum_minutes = 0;
run;


************** Calculate the mean and sum of trip miles by mode;

* proc sort;
* by age_group sex mpo;
* run;


** Calculate the mean and sum of trip miles and minutes by AGE and SEX;
Ods trace on;
ODS Listing Close;
ODS Output Domain = MyStat_AGESEX_&modegroup;
* ODS Output Statistics = MyStat_AGESEX_&modegroup;
proc surveymeans nobs sumwgt mean stderr cv data = person_miles_minutes_&modegroup;
   title1 "Mean Travel Miles and Times Per Day by &modegroup, Age and Gender, NHTS, California, 2009";
*	by age_group sex mpo;
    domain age_group*sex*&geography;
    var sum_miles sum_minutes;
    weight WTPERFIN; 
run;
ODS OUTPUT CLOSE;
Ods trace off;

** Calculate the mean and sum of trip miles and minutes by RACE;
Ods trace on;
ODS Listing Close;
ODS Output Domain = MyStat_RACE_&modegroup;
* ODS Output Statistics = MyStat_AGESEX_&modegroup;
proc surveymeans nobs sumwgt mean stderr cv data = person_miles_minutes_&modegroup;
   title1 "Mean Travel Miles and Times Per Day by &modegroup, Age and Gender, NHTS, California, 2009";
    domain race*&geography;
    var sum_miles sum_minutes;
    weight WTPERFIN; 
run;
ODS OUTPUT CLOSE;
Ods trace off;

** Calculate the mean and sum of trip miles and minutes by ALL AGES;
Ods trace on;
ODS Listing Close; 
ODS Output domain = MyStat_&modegroup;
proc surveymeans nobs sumwgt mean stderr cv data = person_miles_minutes_&modegroup;
   title1 "Mean Travel Miles and Times Per Day by &modegroup NHTS, California, 2009";
	* by mpo;
	domain &geography;
    var sum_miles sum_minutes;
    weight WTPERFIN; 
run;
ODS OUTPUT CLOSE;
Ods trace off;





/*
* Travel Distance Means by age and sex using survey design;
proc surveymeans; 
   title1 "Mean Travel Miles Per Day by &modegroup, NHTS, California, 2009";
    var sum_miles; 
    weight WTPERFIN; 
run;

* Mean Travel Time per Day by age and sex using survey design;
proc surveymeans nobs sumwgt mean stderr cv; 
   title1 "Mean Travel Times by &modegroup, NHTS, California, 2009";
    var sum_minutes; 
    weight WTPERFIN; 
run;

*/

%let i=%eval(&i+1);
%let modegroup=%scan(&modelist,&i);
%end;
%mend loop1;
%loop1(CAR BUS RAIL WALK BICYCLE MOTORCYCLE OTHER); 



********** Create output tables by mode ****************; 
%macro loop2(modelist);
** Calculate the mean and sum of trip miles and minutes by age and sex;
%local i modegroup;
%let i=1;
%let modegroup=%scan(&modelist,&i);

%do %while (&modegroup ne); 

data mystat_AGESEX_&modegroup;
  set mystat_AGESEX_&modegroup;
  mode_strata = "&modegroup";
  geotype = "&geography";
  race="All Races";
 run;

data mystat_RACE_&modegroup;
  set mystat_RACE_&modegroup;
	age_group="All Ages";
    sex="All";
	mode_strata = "&modegroup";
	geotype = "&geography";
 run;

 data mystat_&modegroup;
  set mystat_&modegroup;
	age_group="All Ages";
    sex="All";
	mode_strata = "&modegroup";
	geotype = "&geography";
	race="All Races";
 run;



%let i=%eval(&i+1);
%let modegroup=%scan(&modelist,&i);
%end;


** Calculate the mean and sum of trip miles and minutes by ALL AGES;
%local i modegroup;
%let i=1;
%let modegroup=%scan(&modelist,&i);

%do %while (&modegroup ne);
 data mystat_&modegroup;
  set mystat_&modegroup;
  mode_strata = "&modegroup";
  geotype = "&geography";
 run;
%let i=%eval(&i+1);
%let modegroup=%scan(&modelist,&i);
%end;

%mend loop2;
%loop2(CAR BUS RAIL WALK BICYCLE MOTORCYCLE OTHER);


************* Concatenate the tables for items 1 and 2; 
Data item_&geography;
format mode_strata $CHAR12. DomainLabel $CHAR12. geotype $CHAR12. geoname;
 set mystat_walk mystat_bicycle mystat_car mystat_bus mystat_rail mystat_motorcycle mystat_other mystat_RACE_walk mystat_RACE_bicycle mystat_RACE_car mystat_RACE_bus mystat_RACE_rail mystat_RACE_motorcycle mystat_RACE_other mystat_AGESEX_walk mystat_AGESEX_bicycle mystat_AGESEX_car mystat_AGESEX_bus mystat_AGESEX_rail mystat_AGESEX_motorcycle mystat_AGESEX_other;
 run;

Data item_&geography;
format geotype_name $CHAR26.;
set item_&geography (RENAME =(&geography=geotype_name));
run;



 Data item_&geography;
 set item_&geography (DROP=DomainLabel);
 run;


proc datasets;
delete mystat_walk mystat_bicycle mystat_car mystat_bus mystat_rail mystat_motorcycle mystat_other mystat_RACE_walk mystat_RACE_bicycle mystat_RACE_car mystat_RACE_bus mystat_RACE_rail mystat_RACE_motorcycle mystat_RACE_other mystat_AGESEX_walk mystat_AGESEX_bicycle mystat_AGESEX_car mystat_AGESEX_bus mystat_AGESEX_rail mystat_AGESEX_motorcycle mystat_AGESEX_other;
run;


proc sort; by mode_strata; run;


proc datasets;
delete TRIPSCAR COLLAPSED_SUM_CAR PERSON_MILES_MINUTES_CAR TRIPSBUS COLLAPSED_SUM_BUS PERSON_MILES_MINUTES_BUS TRIPSRAIL COLLAPSED_SUM_RAIL PERSON_MILES_MINUTES_RAIL TRIPSWALK COLLAPSED_SUM_WALK PERSON_MILES_MINUTES_WALK TRIPSBICYCLE COLLAPSED_SUM_BICYCLE PERSON_MILES_MINUTES_BICYCLE TRIPSMOTORCYCLE COLLAPSED_SUM_MOTORCYCLE PERSON_MILES_MINUTES_MOTORCYCLE TRIPSOTHER COLLAPSED_SUM_OTHER PERSON_MILES_MINUTES_OTHER;
run;


/*
proc import datafile='X:\ITHIM\Crosswalks\CA_&geography2010.xls'  dbms=xls out= CA_&geography2010 replace;
getnames=yes;
run;
*/


%let j=%eval(&j+1);
%let geography=%scan(&geolist,&j);
%end;
%mend geoloop;
%geoloop(county mpo megaregion);





************ Item 1;
Data item1;
retain item_id item_name units urbrur report_year geotype geotype_name geotype_code mode_strata age_group sex race n sumwgt mean stderr cv version;
set item_county item_mpo item_megaregion;



if VarName = "sum_minutes";

 
 item_id = 1;
 Item_name  = "Per capita mean daily travel time";
 units = "min/person/d";
 urbrur = 1;
 geotype_code=0;
 strata_type='race'; 
 *age_group = 'All Ages';
 report_year = "2009";
 version = DATETIME();
 format version datetime16.0;
run;


data item1; 
 set item1 (drop = VarName);
run;
proc contents;run;


data modecode_lookup;
	format mode $CHAR12.;
	input mode $ mode_strata_code;	
	DATALINES;
BICYCLE 1
WALK 2
CAR 3
BUS 4
RAIL 5
MOTORCYCLE 6
OTHER 7
;
proc print;
run;



proc sql;
CREATE TABLE item1 AS
select 
&scenario AS scenario_id, 
A.item_id, A.item_name, 
A.units AS units, 
A.urbrur, 
A.report_year, 
A.geotype, 
A.geotype_name, 
A.mode_strata as mode, 
B.mode_strata_code as mode_code, 
A.age_group AS age_group, 
A.sex, 
A.strata_type, 
A.race as strata, 
A.n AS unwt_n, 
A.sumwgt AS wt_n, 
A.mean AS item_result,
A.stderr AS se, 
A.cv, 
A.version AS date 
from item1 A INNER JOIN MODECODE_LOOKUP B ON (A.mode_strata=B.mode);
run;

PROC SQL;
CREATE TABLE item1 AS
select A.*, B.geo_code as geotype_code
from item1 A INNER JOIN CA_Geo2010 B ON (A.geotype_name=B.geo_name AND A.geotype=B.geo_type);
run;




************ Item 3;
Data item3 ;
retain item_id item_name units urbrur report_year geotype geotype_name geotype_code mode_strata age_group sex race n sumwgt mean stderr cv version;
 set item_county item_mpo item_megaregion;

if VarName = "sum_miles";

 item_id = 3;
 Item_name  = "per capita mean daily travel distance";
 units = "miles/person/day";
 urbrur = 1;
 strata_type='race';
 geotype_code=0;
 *age_group = 'All Ages';
 report_year = "2009";
 version = DATETIME();
 format version datetime16.0;
run;

data item3; 
 set item3 (drop = VarName);
run;
proc contents;run;

proc sql;
CREATE TABLE item3 AS
select &scenario AS scenario_id, A.item_id, A.item_name, A.units AS units, A.urbrur, A.report_year, 
A.geotype, A.geotype_name, A.mode_strata as mode, B.mode_strata_code as mode_code, A.age_group AS age_group, 
A.sex, A.strata_type, A.race as strata, A.n AS unwt_n, A.sumwgt AS wt_n, A.mean AS item_result, A.stderr AS se, A.cv, 
A.version AS date from item3 A INNER JOIN MODECODE_LOOKUP B ON (A.mode_strata=B.mode);
run;

PROC SQL;
CREATE TABLE item3 AS
select A.*, B.geo_code as geotype_code
from item3 A INNER JOIN CA_Geo2010 B ON (A.geotype_name=B.geo_name AND A.geotype=B.geo_type);
run;


proc sql;
CREATE TABLE female15_29 AS
SELECT mode, 
mode_code, 
item_result,
geotype,
geotype_name
FROM item1
WHERE mode_code in (1,2)and 
sex = 'F' and 
age_group = '15-29';
run; 

proc sql;
CREATE TABLE item2 AS
SELECT &scenario AS scenario_id, 2 as item_id,
'Ratio of per capita mean daily active travel time (relative to females aged 15-29 years old)' AS item_name,
'Dimensionless' AS units,
A.urbrur, 
A.report_year,
A.geotype,
A.geotype_name,
C.geo_code as geotype_code,
A.mode,
A.mode_code,
A.age_group,
A.sex, 
'race' as strata_type,
A.strata,
A.unwt_n,
A.wt_n,
A.item_result/B.item_result AS item_result,
A.se,
A.cv, 
A.date
FROM item1 A, female15_29 B, CA_geo2010 C
WHERE A.mode_code = B.mode_code and 
A.geotype_name = B.geotype_name and 
A.geotype = B.geotype and
A.mode_code in (1,2) and
A.strata="All Races" and
A.geotype_name=C.geo_name and
A.geotype=C.geo_type;
run;

proc sql;
create table ITHIM_CalibrationData1to3 AS 
select scenario_id, item_id, item_name, units, urbrur, report_year, geotype, geotype_name, geotype_code, mode, mode_code, age_group, sex, strata_type, strata, unwt_n, wt_n, item_result, se, cv, date
 from item1
union
select scenario_id, item_id, item_name, units, urbrur, report_year, geotype, geotype_name, geotype_code, mode, mode_code, age_group, sex, strata_type, strata, unwt_n, wt_n, item_result, se, cv, date
 from item2
union
select scenario_id, item_id, item_name, units, urbrur, report_year, geotype, geotype_name, geotype_code, mode, mode_code, age_group, sex, strata_type, strata, unwt_n, wt_n, item_result, se, cv, date
 from item3;
run;

/*
proc sql;
create table SANDAG_ITHIM_INPUT AS
select * from ITHIM_INPUT
WHERE geotype_code = 'San Diego';
run;
*/

proc export
data=ITHIM_CalibrationData1to3 outfile="T:\ITHIM\CalibrationData\Item1_PerCapMeanDailyTT\ITHIM_Items1to3_1-13-15.csv" dbms=csv replace;
run;

proc export
data=ITHIM_CalibrationData1to3 outfile="T:\ITHIM\CalibrationData\Item1_PerCapMeanDailyTT\ITHIM_Items1to3_1-13-15.csv.xls" dbms=xls replace;
run;

/*
proc export data=SANDAG_ITHIM_INPUT outfile="\\phdeorlcsrvip01\Crossbranch\ITHIM\Outputs\SANDAG_ITHIM_Items1to3_&geography..csv" dbms=csv replace;
*data=SANDAG_ITHIM_INPUT outfile="D:\Brandon\ITHIM\SANDAG_ITHIM_INPUT.csv" dbms=csv replace;
run;
*/
*/